@CHARSET "UTF-8";
 
html, body {
    height: 100%;
    background-color: #000 !important;
}

h2 {
    font-family: NotesEsaBold, "Trebuchet MS", Tahoma, Arial;
    font-size: 18px;
    line-height: 15px;
    font-weight: bold;
    text-transform: uppercase;
    color: #3277a6;
    text-decoration: none;
    margin: 0;
    padding: 0;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -o-user-select: none;
    user-select: none;
    cursor: default
}

#loading {
    text-align: center;
    font-size: small !important;
}

#reloadMessage {
    text-align: center;
    font-size: medium !important;
    margin-top: 10px;
}

#esa-logo {
    position: absolute;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
}

#esa-logo-image{
	display: block;
	margin: auto;
}

.esasky__spinner{width:32px;height:30px;display: inline-block;vertical-align:middle;}.esasky__spinner div{animation:esasky__spinner 1.5s cubic-bezier(.5,0,.5,1) infinite;transform-origin:15px 15px}.esasky__spinner div:after{content:" ";display:block;position:absolute;width:4px;height:4px;border-radius:50%;background:#fff;margin:-3px 0 0 -3px;top:4px;left:17px}@keyframes esasky__spinner{0%{transform:rotate(0)}100%{transform:rotate(360deg)}}
.esasky__spinner div:nth-child(1), .esasky__spinner_large div:nth-child(1) {
	animation-delay: -0.4s;
}
.esasky__spinner div:nth-child(2), .esasky__spinner__large div:nth-child(2) {
	animation-delay: -0.30s;
}
.esasky__spinner div:nth-child(3), .esasky__spinner__large div:nth-child(3) {
	animation-delay: -0.20s;
}
.esasky__spinner div:nth-child(4), .esasky__spinner__large div:nth-child(4) {
	animation-delay: -0.10s;
}