var serverProperties = {
 contextPath: "${context.path}",
 tapContext: "https://sky.esa.int/esasky-tap",
 localeFilesLocation: "",
 targetListFilesLocation: "",
 showExtTap: "true",
 showMissingTranslationBox: "false"
};